@@ -14,9 +14,6 @@ const Util = imports.misc.util;
 const Settings = imports.ui.settings;
 const Signals = imports.signals;
 
-const DEFAULT_ICON_SIZE = 20;
-const ICON_HEIGHT_FACTOR = 0.8;
-
 const PANEL_EDIT_MODE_KEY = 'panel-edit-mode';
 const PANEL_LAUNCHERS_KEY = 'panel-launchers';
 
@@ -24,18 +21,11 @@ const CUSTOM_LAUNCHERS_PATH = GLib.get_home_dir() + '/.cinnamon/panel-launchers'
 
 let pressLauncher = null;
 
-function PanelAppLauncherMenu(launcher, orientation) {
-    this._init(launcher, orientation);
-}
-
-PanelAppLauncherMenu.prototype = {
-    __proto__: Applet.AppletPopupMenu.prototype,
-
-    _init: function(launcher, orientation) {
+class PanelAppLauncherMenu extends Applet.AppletPopupMenu {
+    constructor(launcher, orientation) {
+        super(launcher, orientation);
         this._launcher = launcher;
 
-        Applet.AppletPopupMenu.prototype._init.call(this, launcher, orientation);
-
         let appinfo = this._launcher.getAppInfo();
 
         this._actions = appinfo.list_actions();
@@ -82,45 +72,40 @@ PanelAppLauncherMenu.prototype = {
             AppletManager._removeAppletFromPanel(this._launcher._applet._uuid, this._launcher._applet.instance_id);
         }));
         subMenu.menu.addMenuItem(item);
-    },
+    }
 
-    _onLaunchActivate: function(item, event) {
+    _onLaunchActivate(item, event) {
         this._launcher.launch();
-    },
+    }
 
-    _onRemoveActivate: function(item, event) {
+    _onRemoveActivate(item, event) {
         this.close();
         this._launcher.launchersBox.removeLauncher(this._launcher, this._launcher.isCustom());
         this._launcher.actor.destroy();
-    },
+    }
 
-    _onAddActivate: function(item, event) {
+    _onAddActivate(item, event) {
         this._launcher.launchersBox.showAddLauncherDialog(event.get_time());
-    },
+    }
 
-    _onEditActivate: function(item, event) {
+    _onEditActivate(item, event) {
         this._launcher.launchersBox.showAddLauncherDialog(event.get_time(), this._launcher);
-    },
+    }
 
-    _launchAction: function(event, name) {
+    _launchAction(event, name) {
         this._launcher.launchAction(name);
     }
-};
-
-function PanelAppLauncher(launchersBox, app, appinfo, orientation, panel_height, scale) {
-    this._init(launchersBox, app, appinfo, orientation, panel_height, scale);
 }
 
-PanelAppLauncher.prototype = {
-    __proto__: DND.LauncherDraggable.prototype,
-
-    _init: function(launchersBox, app, appinfo, orientation, panel_height, scale) {
-        DND.LauncherDraggable.prototype._init.call(this);
+class PanelAppLauncher extends DND.LauncherDraggable {
+    constructor(launchersBox, app, appinfo, orientation, icon_size) {
+        super();
         this.app = app;
         this.appinfo = appinfo;
         this.launchersBox = launchersBox;
         this._applet = launchersBox;
         this.orientation = orientation;
+        this.icon_size = icon_size;
 
         this.actor = new St.Bin({ style_class: 'launcher',
                                   important: true,
@@ -136,21 +121,18 @@ PanelAppLauncher.prototype = {
 
         this._iconBox = new St.Bin({ style_class: 'icon-box',
                                      important: true });
-        this._iconBox.connect('style-changed',
-                              Lang.bind(this, this._onIconBoxStyleChanged));
-        this._iconBox.connect('notify::allocation',
-                              Lang.bind(this, this._updateIconBoxClip));
+
         this.actor.add_actor(this._iconBox);
         this._iconBottomClip = 0;
 
-        if (scale) {
-            this.icon_height = Math.floor((panel_height * ICON_HEIGHT_FACTOR) / global.ui_scale);
-        } else {
-            this.icon_height = DEFAULT_ICON_SIZE;
-        }
         this.icon = this._getIconActor();
         this._iconBox.set_child(this.icon);
 
+        this._iconBox.connect('style-changed',
+                              Lang.bind(this, this._updateIconSize));
+        this._iconBox.connect('notify::allocation',
+                              Lang.bind(this, this._updateIconSize));
+
         this._menuManager = new PopupMenu.PopupMenuManager(this);
         this._menu = new PanelAppLauncherMenu(this, orientation);
         this._menuManager.addMenu(this._menu);
@@ -168,53 +150,53 @@ PanelAppLauncher.prototype = {
         this._updateInhibit();
         this.launchersBox.connect("launcher-draggable-setting-changed", Lang.bind(this, this._updateInhibit));
         global.settings.connect('changed::' + PANEL_EDIT_MODE_KEY, Lang.bind(this, this._updateInhibit));
-    },
+    }
 
-    _onDragBegin: function() {
+    _onDragBegin() {
         this._dragging = true;
         this._tooltip.hide();
         this._tooltip.preventShow = true;
-    },
+    }
 
-    _onDragEnd: function() {
+    _onDragEnd() {
         this._dragging = false;
         this._tooltip.preventShow = false;
         this._applet._clearDragPlaceholder();
-    },
+    }
 
-    _onDragCancelled: function() {
+    _onDragCancelled() {
         this._dragging = false;
         this._tooltip.preventShow = false;
-    },
+    }
 
-    _updateInhibit: function(){
+    _updateInhibit() {
         let editMode = global.settings.get_boolean(PANEL_EDIT_MODE_KEY);
         this._draggable.inhibit = !this.launchersBox.allowDragging || editMode;
         this.actor.reactive = !editMode;
-    },
+    }
 
-    getDragActor: function() {
+    getDragActor() {
         return this._getIconActor();
-    },
+    }
 
     // Returns the original actor that should align with the actor
     // we show as the item is being dragged.
-    getDragActorSource: function() {
+    getDragActorSource() {
         return this.icon;
-    },
+    }
 
-    _getIconActor: function() {
+    _getIconActor() {
         if (this.isCustom()) {
             let icon = this.appinfo.get_icon();
             if (icon == null)
                 icon = new Gio.ThemedIcon({name: "gnome-panel-launcher"});
-            return new St.Icon({gicon: icon, icon_size: this.icon_height, icon_type: St.IconType.FULLCOLOR});
+            return new St.Icon({gicon: icon, icon_size: this.icon_size, icon_type: St.IconType.FULLCOLOR});
         } else {
-            return this.app.create_icon_texture(this.icon_height);
+            return this.app.create_icon_texture(this.icon_size);
         }
-    },
+    }
 
-    _animateIcon: function(step) {
+    _animateIcon(step) {
         if (step >= 3) return;
         this.icon.set_pivot_point(0.5, 0.5);
         Tweener.addTween(this.icon,
@@ -222,13 +204,13 @@ PanelAppLauncher.prototype = {
                            scale_y: 0.7,
                            time: 0.2,
                            transition: 'easeOutQuad',
-                           onComplete: function() {
+                           onComplete() {
                                Tweener.addTween(this.icon,
                                                 { scale_x: 1.0,
                                                   scale_y: 1.0,
                                                   time: 0.2,
                                                   transition: 'easeOutQuad',
-                                                  onComplete: function() {
+                                                  onComplete() {
                                                       this._animateIcon(step + 1);
                                                   },
                                                   onCompleteScope: this
@@ -236,42 +218,40 @@ PanelAppLauncher.prototype = {
                            },
                            onCompleteScope: this
                          });
-    },
+    }
 
-    launch: function() {
-        let allocation = this._iconBox.get_allocation_box();
-        this._iconBox.width = allocation.x2 - allocation.x1;
-        this._iconBox.height = allocation.y2 - allocation.y1;
+    launch() {
+        if (this.isCustom()) {
+            this.appinfo.launch([], null);
+        }
+        else {
+            this.app.open_new_window(-1);
+        }
         this._animateIcon(0);
-        if (this.isCustom()) this.appinfo.launch([], null);
-        else this.app.open_new_window(-1);
-    },
+    }
 
-    launchAction: function(name) {
-        let allocation = this._iconBox.get_allocation_box();
-        this._iconBox.width = allocation.x2 - allocation.x1;
-        this._iconBox.height = allocation.y2 - allocation.y1;
-        this._animateIcon(0);
+    launchAction(name) {
         this.getAppInfo().launch_action(name, null);
-    },
+        this._animateIcon(0);
+    }
 
-    getId: function() {
+    getId() {
         if (this.isCustom()) return Gio.file_new_for_path(this.appinfo.get_filename()).get_basename();
         else return this.app.get_id();
-    },
+    }
 
-    isCustom: function() {
+    isCustom() {
         return (this.app==null);
-    },
+    }
 
-    _onButtonPress: function(actor, event) {
+    _onButtonPress(actor, event) {
         pressLauncher = this.getAppname();
 
         if (event.get_button() == 3)
             this._menu.toggle();
-    },
+    }
 
-    _onButtonRelease: function(actor, event) {
+    _onButtonRelease(actor, event) {
         if (pressLauncher == this.getAppname()){
             let button = event.get_button();
             if (button==1) {
@@ -279,35 +259,32 @@ PanelAppLauncher.prototype = {
                 else this.launch();
             }
         }
-    },
+    }
 
-    _onIconBoxStyleChanged: function() {
+    _updateIconSize() {
         let node = this._iconBox.get_theme_node();
-        this._iconBottomClip = node.get_length('panel-launcher-bottom-clip');
-        this._updateIconBoxClip();
-    },
+        let maxHeight = this._iconBox.height - node.get_vertical_padding();
+        let maxWidth = this._iconBox.width - node.get_horizontal_padding();
+        let smallestDim = Math.min(maxHeight, maxWidth) / global.ui_scale;
 
-    _updateIconBoxClip: function() {
-        let allocation = this._iconBox.allocation;
-        if (this._iconBottomClip > 0)
-            this._iconBox.set_clip(0, 0, allocation.x2 - allocation.x1, allocation.y2 - allocation.y1 - this._iconBottomClip);
-        else
-            this._iconBox.remove_clip();
-    },
+        if (smallestDim < this.icon.get_icon_size()) {
+            this.icon.set_icon_size(smallestDim);
+        }
+    }
 
-    getAppInfo: function() {
+    getAppInfo() {
         return (this.isCustom() ? this.appinfo : this.app.get_app_info());
-    },
+    }
 
-    getCommand: function() {
+    getCommand() {
         return this.getAppInfo().get_commandline();
-    },
+    }
 
-    getAppname: function() {
+    getAppname() {
         return this.getAppInfo().get_name();
-    },
+    }
 
-    getIcon: function() {
+    getIcon() {
         let icon = this.getAppInfo().get_icon();
         if (icon) {
             if (icon instanceof Gio.FileIcon) {
@@ -319,22 +296,17 @@ PanelAppLauncher.prototype = {
         }
         return null;
     }
-};
-
-function MyApplet(metadata, orientation, panel_height, instance_id) {
-    this._init(metadata, orientation, panel_height, instance_id);
 }
 
-MyApplet.prototype = {
-    __proto__: Applet.Applet.prototype,
-
-    _init: function(metadata, orientation, panel_height, instance_id) {
-        Applet.Applet.prototype._init.call(this, orientation, panel_height, instance_id);
+class CinnamonPanelLaunchersApplet extends Applet.Applet {
+    constructor(metadata, orientation, panel_height, instance_id) {
+        super(orientation, panel_height, instance_id);
         this.actor.set_track_hover(false);
 
         this.setAllowedLayout(Applet.AllowedLayout.BOTH);
 
         this.orientation = orientation;
+        this.icon_size = this.getPanelIconSize(St.IconType.FULLCOLOR);
         this._dragPlaceholder = null;
         this._dragPlaceholderPos = -1;
         this._animatingPlaceholdersCount = 0;
@@ -358,34 +330,34 @@ MyApplet.prototype = {
         this.do_gsettings_import();
 
         this.on_orientation_changed(orientation);
-    },
+    }
 
-    _updateLauncherDrag: function() {
+    _updateLauncherDrag() {
         this.emit("launcher-draggable-setting-changed");
-    },
+    }
 
-    do_gsettings_import: function() {
+    do_gsettings_import() {
         let old_launchers = global.settings.get_strv(PANEL_LAUNCHERS_KEY);
         if (old_launchers.length >= 1 && old_launchers[0] != "DEPRECATED") {
             this.launcherList = old_launchers;
         }
 
         global.settings.set_strv(PANEL_LAUNCHERS_KEY, ["DEPRECATED"]);
-    },
+    }
 
-    _onPanelEditModeChanged: function() {
+    _onPanelEditModeChanged() {
         this.actor.reactive = global.settings.get_boolean(PANEL_EDIT_MODE_KEY);
-    },
+    }
 
-    _onSettingsChanged: function() {
+    _onSettingsChanged() {
         this.reload();
-    },
+    }
 
-    sync_settings_proxy_to_settings: function() {
+    sync_settings_proxy_to_settings() {
         this.launcherList = this._settings_proxy.map(x => x.file);
-    },
+    }
 
-    _remove_launcher_from_proxy: function(visible_index) {
+    _remove_launcher_from_proxy(visible_index) {
         let j = -1;
         for (let i = 0; i < this._settings_proxy.length; i++) {
             if (this._settings_proxy[i].valid) {
@@ -396,9 +368,9 @@ MyApplet.prototype = {
                 }
             }
         }
-    },
+    }
 
-    _move_launcher_in_proxy: function(launcher, new_index) {
+    _move_launcher_in_proxy(launcher, new_index) {
         let proxy_member;
 
         for (let i = 0; i < this._settings_proxy.length; i++) {
@@ -424,22 +396,28 @@ MyApplet.prototype = {
 
         if (new_index == j + 1)
             this._settings_proxy.push(proxy_member);
-    },
+    }
 
-    loadSingleApp: function(path) {
+    loadSingleApp(path) {
         let appSys = Cinnamon.AppSystem.get_default();
         let app = appSys.lookup_app(path);
         let appinfo = null;
         if (!app)
             appinfo = Gio.DesktopAppInfo.new_from_filename(CUSTOM_LAUNCHERS_PATH+"/"+path);
         return [app, appinfo];
-    },
+    }
 
-    on_panel_height_changed: function() {
+    on_panel_height_changed() {
+        this.icon_size = this.getPanelIconSize(St.IconType.FULLCOLOR);
         this.reload();
-    },
+    }
 
-    on_orientation_changed: function(neworientation) {
+    on_panel_icon_size_changed(size) {
+        this.icon_size = size;
+        this.reload();
+    }
+
+    on_orientation_changed(neworientation) {
         this.orientation = neworientation;
         if (this.orientation == St.Side.TOP || this.orientation == St.Side.BOTTOM) {
             this.myactor.remove_style_class_name('vertical');
@@ -453,9 +431,9 @@ MyApplet.prototype = {
             this.myactor.set_y_expand(false);
         }
         this.reload();
-    },
+    }
 
-    reload: function() {
+    reload() {
         this.myactor.destroy_all_children();
         this._launchers = [];
         this._settings_proxy = [];
@@ -464,8 +442,7 @@ MyApplet.prototype = {
             let [app, appinfo] = this.loadSingleApp(file);
 
             if (app || appinfo) {
-                let launcher = new PanelAppLauncher(this, app, appinfo,
-                        this.orientation, this._panelHeight, this._scaleMode);
+                let launcher = new PanelAppLauncher(this, app, appinfo, this.orientation, this.icon_size);
                 this.myactor.add(launcher.actor);
                 this._launchers.push(launcher);
 
@@ -475,9 +452,9 @@ MyApplet.prototype = {
             }
         }
 
-    },
+    }
 
-    removeLauncher: function(launcher, delete_file) {
+    removeLauncher(launcher, delete_file) {
         let i = this._launchers.indexOf(launcher);
         if (i >= 0) {
             launcher.actor.destroy();
@@ -491,36 +468,36 @@ MyApplet.prototype = {
         }
 
         this.sync_settings_proxy_to_settings();
-    },
+    }
 
-    getDummyLauncher: function(path) {
+    getDummyLauncher(path) {
         let [app, appinfo] = this.loadSingleApp(path);
         let dummy;
         if (app || appinfo) {
-            dummy = new PanelAppLauncher(this, app, appinfo, this.orientation, this._panelHeight);
+            dummy = new PanelAppLauncher(this, app, appinfo, this.orientation, this.icon_size);
         }
 
         if (dummy && dummy.actor)
             return dummy.actor;
         else
             return null;
-    },
+    }
 
-    acceptNewLauncher: function(path) {
+    acceptNewLauncher(path) {
         this.myactor.add(this.getDummyLauncher(path));
         let launchers = this.launcherList;
         launchers.push(path);
         this.launcherList = launchers;
         this.reload();
-    },
+    }
 
-    addForeignLauncher: function(path, position, source) {
+    addForeignLauncher(path, position, source) {
         this.myactor.insert_child_at_index(this.getDummyLauncher(path), position);
         this._settings_proxy.splice(position, 0, { file: path, valid: true });
         this.sync_settings_proxy_to_settings();
-    },
+    }
 
-    moveLauncher: function(launcher, pos) {
+    moveLauncher(launcher, pos) {
         let origpos = this._launchers.indexOf(launcher);
         if (origpos >= 0) {
             launcher.actor.destroy();
@@ -530,25 +507,25 @@ MyApplet.prototype = {
             this.sync_settings_proxy_to_settings();
             this.reload(); // overkill really, but a way of getting the scaled size right
         }
-    },
+    }
 
-    showAddLauncherDialog: function(timestamp, launcher){
+    showAddLauncherDialog(timestamp, launcher){
         if (launcher) {
             Util.spawnCommandLine("cinnamon-desktop-editor -mcinnamon-launcher -f" + launcher.getId() + " " + this.settings.file.get_path());
         } else {
             Util.spawnCommandLine("cinnamon-desktop-editor -mcinnamon-launcher " + this.settings.file.get_path());
         }
-    },
+    }
 
-    _clearDragPlaceholder: function() {
+    _clearDragPlaceholder() {
         if (this._dragPlaceholder) {
             this._dragPlaceholder.animateOutAndDestroy();
             this._dragPlaceholder = null;
             this._dragPlaceholderPos = -1;
         }
-    },
+    }
 
-    handleDragOver: function(source, actor, x, y, time) {
+    handleDragOver(source, actor, x, y, time) {
         if (!(source.isDraggableApp || (source instanceof DND.LauncherDraggable))) return DND.DragMotionResult.NO_DROP;
         let children = this.myactor.get_children();
         let numChildren = children.length;
@@ -597,7 +574,7 @@ MyApplet.prototype = {
                     this._animatingPlaceholdersCount++;
                     this._dragPlaceholder.actor.connect('destroy',
                         Lang.bind(this, function() {
-                        this._animatingPlaceholdersCount--;
+                            this._animatingPlaceholdersCount--;
                         }));
                 }
                 this._dragPlaceholder = null;
@@ -628,9 +605,9 @@ MyApplet.prototype = {
             return DND.DragMotionResult.MOVE_DROP;
 
         return DND.DragMotionResult.COPY_DROP;
-    },
+    }
 
-    acceptDrop: function(source, actor, x, y, time) {
+    acceptDrop(source, actor, x, y, time) {
         if (!(source.isDraggableApp || (source instanceof DND.LauncherDraggable))) return DND.DragMotionResult.NO_DROP;
 
         let sourceId;
@@ -658,10 +635,9 @@ MyApplet.prototype = {
         actor.destroy();
         return true;
     }
-};
-Signals.addSignalMethods(MyApplet.prototype);
+}
+Signals.addSignalMethods(CinnamonPanelLaunchersApplet.prototype);
 
 function main(metadata, orientation, panel_height, instance_id) {
-    let myApplet = new MyApplet(metadata, orientation, panel_height, instance_id);
-    return myApplet;
+    return new CinnamonPanelLaunchersApplet(metadata, orientation, panel_height, instance_id);
 }
